%framework from Serge Gale

function C = C_hat(q,dq)
%Finds the estimation of C(q,dq) by using RBFNN with pruning

global m2;
global l1 l2;
global Cmx2 Cmy2;

C_11 = -dq(2) * m2 * l1 * (Cmx2 * sin(q(2)) + Cmy2 * cos(q(2)) + sin(q(2)) * l2);
C_12 = -m2 * l1 * (Cmx2 * sin(q(2)) + Cmy2 * cos(q(2)) + sin(q(2)) * l2) * (dq(1) + dq(2));
C_21 = dq(1) * m2 * l1 * (Cmx2 * sin(q(2)) + Cmy2 * cos(q(2)) + sin(q(2)) * l2);
C_22 = 0;

%RBF networks
global RBFNN_C_11 RBFNN_C_12 RBFNN_C_21;
%stores iteration number, p_thW, prunedW, p_thO, prunedO
global C_11_prun C_12_prun C_21_prun;
%stores the network's estimation errors
global DATA_C;
%activation function output ratio sums
global act_outp_C_11 act_outp_C_12 act_outp_C_21; 
%pruning thresholds  
global weight_th outp_th_C;
%counter for number of times network pruned by weight pruning
global n_prunW_C;


C_11_est  = RBFNN_C_11.Calculate([q(2) dq(2)]);
C_12_est = RBFNN_C_12.Calculate([q(2) (dq(2) + dq(1))]);
C_21_est = RBFNN_C_21.Calculate([q(2) dq(1)]);

C_11_err = C_11 - C_11_est;
C_12_err = C_12 - C_12_est;
C_21_err = C_21 - C_21_est;

DATA_C = [DATA_C, [C_11_err; C_12_err; C_21_err]];

RBFNN_C_11 = RBFNN_C_11.Train([q(2) dq(2)], C_11_err);
RBFNN_C_12 = RBFNN_C_12.Train([q(2) (dq(2) + dq(1))], C_12_err);
RBFNN_C_21 = RBFNN_C_21.Train([q(2) dq(1)], C_21_err);

[C_11_est outp_C_11] = RBFNN_C_11.Calculate([q(2) dq(2)]);
[C_12_est outp_C_12]= RBFNN_C_12.Calculate([q(2) (dq(2) + dq(1))]);
[C_21_est outp_C_21]= RBFNN_C_21.Calculate([q(2) dq(1)]);

act_outp_C_11 = act_outp_C_11 + outp_C_11;
act_outp_C_12 = act_outp_C_12 + outp_C_12;
act_outp_C_21 = act_outp_C_21 + outp_C_21;


if length(C_11_prun(1,:)) == 1
    C_11_prun_1 = 2;
else
    C_11_prun_1 = length(C_11_prun(1,:)) + 1;%iteration number
end

len = length(DATA_C);
%to avoid networks being to early pruned
if len > 1000 
    
    %finding approximation errors for the last 400 iterations.
    d1 = DATA_C(1, len-400:len);
    d2 = DATA_C(2, len-400:len);
    d3 = DATA_C(3, len-400:len);
    rms_C11 = Rmse(d1);
    rms_C12 = Rmse(d2);
    rms_C21 = Rmse(d3);

    
    if rms_C11 < 0.01
        %prune by node output
        [RBFNN_C_11 act_outp_C_11 prunedO p_thO] = RBFNN_C_11.pruneO(act_outp_C_11, outp_th_C, 2);
        %prune by weight magnitude
        [RBFNN_C_11 n_prunC11 prunedW p_thW act_outp_C_11] = RBFNN_C_11.pruneW(weight_th, 2, n_prunW_C(1,1), act_outp_C_11);
        n_prunW_C(1,1) = n_prunC11;
        
        %store the pruning result
        C_11_prun_2 =  p_thW;
        if prunedW == 0 
            C_11_prun_3 = 0; 
        else
            C_11_prun_3 = length(prunedW);%units pruned by weight 
        end
        C_11_prun_4 =  p_thO;
        if prunedO == 0 
            C_11_prun_5 = 0; 
        else
            C_11_prun_5 = length(prunedO);%units pruned by node 
        end
        C_11_prun = [C_11_prun, [C_11_prun_1; C_11_prun_2; C_11_prun_3;C_11_prun_4;C_11_prun_5]];
    else%if the error is not small enough
        C_11_prun_2 = 0;
        C_11_prun_3 = 0;
        C_11_prun_4 = 0;
        C_11_prun_5 = 0;
        C_11_prun = [C_11_prun, [C_11_prun_1; C_11_prun_2; C_11_prun_3; C_11_prun_4; C_11_prun_5]];
    end
    
    
    if rms_C12 < 0.01
        %prune by node output
        [RBFNN_C_12 act_outp_C_12 prunedO p_thO] = RBFNN_C_12.pruneO(act_outp_C_12, outp_th_C, 2);
        %prune by weight magnitude
        [RBFNN_C_12 n_prunC12  prunedW p_thW act_outp_C_12] = RBFNN_C_12.pruneW(weight_th, 2, n_prunW_C(2,1), act_outp_C_12);
        n_prunW_C(2,1) = n_prunC12;       
 
        %store the pruning result
        C_12_prun_2 =  p_thW;
        if prunedW == 0
            C_12_prun_3 = 0; 
        else
            C_12_prun_3 = length(prunedW);
        end
        C_12_prun_4 =  p_thO;
        if prunedO == 0 
            C_12_prun_5 = 0; 
        else
            C_12_prun_5 = length(prunedO);
        end
        C_12_prun = [C_12_prun, [C_11_prun_1; C_12_prun_2; C_12_prun_3; C_12_prun_4; C_12_prun_5]];
        
    else%if the error is not small enough
        C_12_prun_2 = 0;
        C_12_prun_3 = 0;
        C_12_prun_4 = 0;
        C_12_prun_5 = 0;
        C_12_prun = [C_11_prun, [C_11_prun_1; C_12_prun_2; C_12_prun_3; C_12_prun_4; C_12_prun_5]];       
    end
    
    if rms_C21 < 0.01
        %prune by node output
        [RBFNN_C_21 act_outp_C_21 prunedO p_thO] = RBFNN_C_21.pruneO(act_outp_C_21, outp_th_C, 2);
        %prune by weight magnitude
        [RBFNN_C_21 n_prunC21 prunedW p_thW act_outp_C_21] = RBFNN_C_21.pruneW(weight_th, 2, n_prunW_C(3,1), act_outp_C_21);
        n_prunW_C(3,1) = n_prunC21;  
        
        %store the pruning result
        C_21_prun_2 =  p_thW;
        if prunedW == 0 
            C_21_prun_3 = 0; 
        else
            C_21_prun_3 = length(prunedW);%units pruned by weight
        end
        C_21_prun_4 =  p_thO;
        if prunedO == 0 
            C_21_prun_5 = 0; 
        else
            C_21_prun_5 = length(prunedO);%units pruned by node
        end
        C_21_prun = [C_21_prun, [C_11_prun_1; C_21_prun_2; C_21_prun_3; C_21_prun_4; C_21_prun_5 ]];
        
    else%if the error is not small enough
        C_21_prun_2 = 0;
        C_21_prun_3 = 0;
        C_21_prun_4 = 0;
        C_21_prun_5 = 0;
        C_21_prun = [C_21_prun, [C_11_prun_1; C_21_prun_2; C_21_prun_3; C_21_prun_4; C_21_prun_5]];
    end
    
else%not enough iteration done to prune
        C_11_prun_2 = 0;
        C_11_prun_3 = 0;
        C_11_prun_4 = 0;
        C_11_prun_5 = 0;
        C_11_prun = [C_11_prun, [C_11_prun_1; C_11_prun_2; C_11_prun_3; C_11_prun_4; C_11_prun_5]];

        C_12_prun_2 = 0;
        C_12_prun_3 = 0;
        C_12_prun_4 = 0;
        C_12_prun_5 = 0;
        C_12_prun = [C_11_prun, [C_11_prun_1; C_12_prun_2; C_12_prun_3; C_12_prun_4; C_12_prun_5]];  
        
        C_21_prun_2 = 0;
        C_21_prun_3 = 0;
        C_21_prun_4 = 0;
        C_21_prun_5 = 0;
        C_21_prun = [C_21_prun, [C_11_prun_1; C_21_prun_2; C_21_prun_3; C_21_prun_4; C_21_prun_5]];
end


C = [C_11_est C_12_est; C_21_est C_22;];


end