%framework from Serge Gale

function G = G_hat(q)
%Finds the estimaion of G(q) by using RBFNN with pruning

global m1 m2;
global Cmx1 Cmx2;
global Cmy1 Cmy2;

G_11 = m1 * (0.981e1 * cos(q(1)) * Cmx1 - 0.981e1 * sin(q(1)) * Cmy1) + m2 * (0.981e1 * (cos(q(1)) * cos(q(2)) - sin(q(1)) * sin(q(2))) * Cmx2 + 0.981e1 * (-cos(q(1)) * sin(q(2)) - sin(q(1)) * cos(q(2))) * Cmy2);
G_12 = m2 * (0.981e1 * (cos(q(1)) * cos(q(2)) - sin(q(1)) * sin(q(2))) * Cmx2 + 0.981e1 * (-cos(q(1)) * sin(q(2)) - sin(q(1)) * cos(q(2))) * Cmy2);

%RBF networks
global RBFNN_G_11 RBFNN_G_12;
%stores iteration number, p_thW, prunedW, p_thO, prunedO
global G_11_prun G_12_prun;
%stores the network's estimation errors
global DATA_G;
%activation function output ratio sums
global act_outp_G_11 act_outp_G_12 ;
%pruning thresholds 
global weight_th outp_th_G;
%counter for number of times network pruned by weight pruning
global n_prunW_G;


G_11_est = RBFNN_G_11.Calculate([q(1) q(2)]);
G_12_est = RBFNN_G_12.Calculate([q(1) q(2)]);

G_11_err = G_11 - G_11_est;
G_12_err = G_12 - G_12_est;

DATA_G = [DATA_G, [G_11_err; G_12_err]];

RBFNN_G_11 = RBFNN_G_11.Train([q(1) q(2)], G_11_err);
RBFNN_G_12 = RBFNN_G_12.Train([q(1) q(2)], G_12_err);

[G_11_est outp_G_11] = RBFNN_G_11.Calculate([q(1) q(2)]);
[G_12_est outp_G_12] = RBFNN_G_12.Calculate([q(1) q(2)]);

act_outp_G_11 = act_outp_G_11 + outp_G_11;
act_outp_G_12 = act_outp_G_12 + outp_G_12;

if length(G_11_prun(1,:)) == 1
    G_11_prun_1 = 2;
else
    G_11_prun_1 = length(G_11_prun(1,:)) + 1;%simulation iteration number
end

len = length(DATA_G);
%to avoid networks being to early pruned
if len > 1000 
    d1 = DATA_G(1, len-400:len);
    d2 = DATA_G(2, len-400:len);
    rms_G11 = Rmse(d1);
    rms_G12 = Rmse(d2);
    
    if rms_G11 < 0.03
        %prune by node output
        [RBFNN_G_11 act_outp_G_11 prunedO p_thO] = RBFNN_G_11.pruneO(act_outp_G_11, outp_th_G, 2);
        %prune by weight magnitude
        [RBFNN_G_11 n_prunG11 prunedW p_thW act_outp_G_11] = RBFNN_G_11.pruneW(weight_th, 2, n_prunW_G(1,1), act_outp_G_11);
        n_prunW_G(1,1) = n_prunG11;

        %store the pruning result
        G_11_prun_2 = p_thW;
        if prunedW == 0 
            G_11_prun_3 = 0; 
        else
            G_11_prun_3 = length(prunedW);%units pruned by weight
        end
        G_11_prun_4 = p_thO;
        if prunedO == 0 
            G_11_prun_5 = 0; 
        else
            G_11_prun_5 = length(prunedO);%units pruned by node output
        end
        G_11_prun = [G_11_prun, [G_11_prun_1; G_11_prun_2; G_11_prun_3; G_11_prun_4; G_11_prun_5]];
        
    else%if the error is not small enough
        G_11_prun_2 = 0;
        G_11_prun_3 = 0;
        G_11_prun_4 = 0;
        G_11_prun_5 = 0;
        G_11_prun = [G_11_prun, [G_11_prun_1; G_11_prun_2; G_11_prun_3; G_11_prun_4; G_11_prun_5]]; 
    end
    
    if rms_G12 < 0.08
        %prune by node output
        [RBFNN_G_12 act_outp_G_12  prunedO p_thO] = RBFNN_G_12.pruneO(act_outp_G_12, outp_th_G, 2);
        %prune by weight magnitude
        [RBFNN_G_12 n_prunG12 prunedW p_thW act_outp_G_12] = RBFNN_G_12.pruneW(weight_th, 2, n_prunW_G(2,1), act_outp_G_12);
        n_prunW_G(2,1) = n_prunG12;
        
        %store the pruning result
        G_12_prun_2 =  p_thW;
        if prunedW == 0 
            G_12_prun_3 = 0; 
        else
            G_12_prun_3 = length(prunedW);
        end
        G_12_prun_4 =  p_thO;
        if prunedO == 0
            G_12_prun_5 = 0; 
        else
            G_12_prun_5 = length(prunedO);
        end
        G_12_prun = [G_12_prun, [G_11_prun_1; G_12_prun_2; G_12_prun_3; G_12_prun_4; G_12_prun_5]];
        
    else%if the error is not small enough
        G_12_prun_2 = 0;
        G_12_prun_3 = 0;
        G_12_prun_4 = 0;
        G_12_prun_5 = 0;
        G_12_prun = [G_12_prun, [G_11_prun_1; G_12_prun_2; G_12_prun_3; G_12_prun_4; G_12_prun_5]]; 
    end
        
else%not done enough iterations
    G_11_prun_2 = 0;
    G_11_prun_3 = 0;
    G_11_prun_4 = 0;
    G_11_prun_5 = 0;
    G_11_prun = [G_11_prun, [G_11_prun_1; G_11_prun_2; G_11_prun_3; G_11_prun_4; G_11_prun_5]];
    
    G_12_prun_2 = 0;
    G_12_prun_3 = 0;
    G_12_prun_4 = 0;
    G_12_prun_5 = 0;
    G_12_prun = [G_12_prun, [G_11_prun_1; G_12_prun_2; G_12_prun_3; G_12_prun_4; G_12_prun_5]]; 
end

G = [G_11_est; G_12_est];

end