%framework from Serge Gale

function M = M_hat(q)
%Finds the estimation of M(q) by using RBFNN with pruning

global m1 m2;
global l1 l2;
global Cmx1 Cmx2;
global Cmy1 Cmy2;
global I1z I2z;

M_11 = (m2 * l2 ^ 2) + I1z + I2z + (m2 * l1 ^ 2) + (2 * m1 * Cmx1 * l1) + (m2 * Cmx2 ^ 2) + m2 * Cmy2 ^ 2 + (2 * m2 * Cmx2 * l2) + 0.2e1 * m2 * l1 * Cmx2 * cos(q(2)) - 0.2e1 * m2 * l1 * Cmy2 * sin(q(2)) + 0.2e1 * m2 * l1 * cos(q(2)) * l2 + (m1 * Cmy1 ^ 2) + (m1 * l1 ^ 2) + (m1 * Cmx1 ^ 2);
M_12 = (m2 * l2 ^ 2) + I2z + (m2 * Cmx2 ^ 2) + m2 * Cmy2 ^ 2 + (2 * m2 * Cmx2 * l2) + m2 * l1 * Cmx2 * cos(q(2)) - m2 * l1 * Cmy2 * sin(q(2)) + m2 * l1 * cos(q(2)) * l2; 
M_21 = M_12;
M_22 = (m2 * l2 ^ 2) + I2z + (m2 * Cmx2 ^ 2) + m2 * Cmy2 ^ 2 + (2 * m2 * Cmx2 * l2);

%RBF networks
global RBFNN_M_11 RBFNN_M_12 RBFNN_M_21 RBFNN_M_22;
%stores the network's estimation errors;
global DATA;
%activation function output ratio sums
global act_outp_M_11 act_outp_M_12 act_outp_M_21 act_outp_M_22;
%pruning thresholds 
global weight_th outp_th;
%counter for number of times network pruned by weight pruning
global n_prunW_M; 
%stores iteration number, p_thW, prunedW, p_thO, prunedO
global M_11_prun M_12_prun M_21_prun M_22_prun;


M_11_est = RBFNN_M_11.Calculate(q(2));
M_12_est = RBFNN_M_12.Calculate(q(2));
M_21_est = M_12_est;
M_22_est = RBFNN_M_22.Calculate(M_22);

M_11_err = M_11 - M_11_est;
M_12_err = M_12 - M_12_est;
M_21_err = M_21 - M_21_est;
M_22_err = M_22 - M_22_est;

DATA = [DATA, [M_11_err; M_12_err; M_21_err; M_22_err]];

RBFNN_M_11 = RBFNN_M_11.Train(q(2), M_11_err);
RBFNN_M_12 = RBFNN_M_12.Train(q(2), M_12_err);
RBFNN_M_21 = RBFNN_M_12;
RBFNN_M_22 = RBFNN_M_22.Train(M_22, M_22_err);

[M_11_est outp_M_11] = RBFNN_M_11.Calculate(q(2));
[M_12_est outp_M_12] = RBFNN_M_12.Calculate(q(2));
M_21_est = M_12_est;
[M_22_est outp_M_22] = RBFNN_M_22.Calculate(M_22);

act_outp_M_11 = act_outp_M_11 + outp_M_11;
act_outp_M_12 = act_outp_M_12 + outp_M_12;
act_outp_M_21 = act_outp_M_12;
act_outp_M_22 = act_outp_M_22 + outp_M_22;


if length(M_11_prun(1,:)) == 1
    M_11_prun_1 = 2;
else
    M_11_prun_1 = length(M_11_prun(1,:)) + 1;%simulation iteration number
end


len = length(DATA);
%to avoid networks being to early pruned
if len > 1000 
    d1 = DATA(1, len-400:len);
    d2 = DATA(2, len-400:len);
    d3 = DATA(3, len-400:len);
    d4 = DATA(4, len-400:len);
    rms_M11 = Rmse(d1);
    rms_M12 = Rmse(d2);
    rms_M21 = Rmse(d3);
    rms_M22 = Rmse(d4);
    
    
    if rms_M11 < 0.01
        %prune by node output
        [RBFNN_M_11 act_outp_M_11 prunedO p_thO] = RBFNN_M_11.pruneO(act_outp_M_11, outp_th, 1);
        %prune by weight magnitude
        [RBFNN_M_11 n_prunM11 prunedW p_thW act_outp_M_11] = RBFNN_M_11.pruneW(weight_th, 1, n_prunW_M(1,1), act_outp_M_11);
        n_prunW_M(1,1) = n_prunM11;
        
       %store the pruning result
        M_11_prun_2 =  p_thW;
        if prunedW == 0
            M_11_prun_3 = 0; 
        else
            M_11_prun_3 = length(prunedW);%units pruned by weight
        end
        M_11_prun_4 =  p_thO;
        if prunedO == 0
            M_11_prun_5 = 0; 
        else
            M_11_prun_5 = length(prunedO);%units pruned node output
        end
        M_11_prun = [M_11_prun, [M_11_prun_1; M_11_prun_2; M_11_prun_3; M_11_prun_4; M_11_prun_5]];
        
    else%if the error is not small enough
        M_11_prun_2 = 0;
        M_11_prun_3 = 0;
        M_11_prun_4 = 0;
        M_11_prun_5 = 0;
        M_11_prun = [M_11_prun, [M_11_prun_1; M_11_prun_2; M_11_prun_3; M_11_prun_4; M_11_prun_5]];
    end
    
    
    if rms_M12 < 0.01
        %prune by node output
        [RBFNN_M_12 act_outp_M_12 prunedO p_thO] = RBFNN_M_12.pruneO(act_outp_M_12, outp_th, 1);
        %prune by weight magnitude
        [RBFNN_M_12 n_prunM12 prunedW p_thW act_outp_M_12] = RBFNN_M_12.pruneW(weight_th, 1, n_prunW_M(2,1), act_outp_M_12);
        n_prunW_M(2,1) = n_prunM12;
        
        %store the pruning result
        M_12_prun_2 =  p_thW;
        if prunedW == 0 
            M_12_prun_3 = 0; 
        else
            M_12_prun_3 = length(prunedW);
        end
        M_12_prun_4 =  p_thO;
        if prunedO == 0 
            M_12_prun_5 = 0; 
        else
            M_12_prun_5 = length(prunedO);
        end        
        M_12_prun = [M_12_prun, [M_11_prun_1; M_12_prun_2; M_12_prun_3; M_12_prun_4; M_12_prun_5]];  
        
    else%if the error is not small enough
        M_12_prun_2 = 0;
        M_12_prun_3 = 0;
        M_12_prun_4 = 0;
        M_12_prun_5 = 0;       
        M_12_prun = [M_12_prun, [M_11_prun_1; M_12_prun_2; M_12_prun_3; M_12_prun_4; M_12_prun_5]];        
    end
    
    if rms_M21 < 0.01
        %prune by node output
        [RBFNN_M_21 act_outp_M_21 prunedO p_thO] = RBFNN_M_21.pruneO(act_outp_M_21, outp_th, 1);
        %prune by weight magnitude
        [RBFNN_M_21 n_prunM21 prunedW p_thW act_outp_M_21] = RBFNN_M_21.pruneW(weight_th, 1, n_prunW_M(3,1), act_outp_M_21);
        n_prunW_M(3,1) = n_prunM21;
        
        %store the pruning result
        M_21_prun_2 =  p_thW;
        if prunedW == 0
            M_21_prun_3 = 0; 
        else
            M_21_prun_3 = length(prunedW);
        end
        M_21_prun_4 =  p_thO;
        if prunedO == 0
            M_21_prun_5 = 0; 
        else
            M_21_prun_5 = length(prunedO);
        end        
        M_21_prun = [M_21_prun, [M_11_prun_1; M_21_prun_2; M_21_prun_3; M_21_prun_4; M_21_prun_5 ]];
        
    else%if the error is not small enough
        M_21_prun_2 = 0;
        M_21_prun_3 = 0;
        M_21_prun_4 = 0;
        M_21_prun_5 = 0;
        M_21_prun = [M_21_prun, [M_11_prun_1; M_21_prun_2; M_21_prun_3; M_21_prun_4; M_21_prun_5 ]];
    end
    
    
    if rms_M22 < 0.01
        %prune by node output
        [RBFNN_M_22 act_outp_M_22 prunedO p_thO] = RBFNN_M_22.pruneO(act_outp_M_22, outp_th, 1);
        %prune by weight magnitude
        [RBFNN_M_22 n_prunM22 prunedW p_thW act_outp_M_22]= RBFNN_M_22.pruneW(weight_th, 1, n_prunW_M(4,1), act_outp_M_22);
        n_prunW_M(4,1) = n_prunM22;
        
        %store the pruning result
        M_22_prun_2 =  p_thW;
        if prunedO == 0
            M_22_prun_3 = 0; 
        else
            M_22_prun_3 = length(prunedW);
        end
        M_22_prun_4 =  p_thO;
        if prunedO == 0
            M_22_prun_5 = 0; 
        else
            M_22_prun_5 = length(prunedO);
        end       
        M_22_prun = [M_22_prun, [M_11_prun_1; M_22_prun_2; M_22_prun_3; M_22_prun_4; M_22_prun_5]];
        
    else%if the error is not small enough
        M_22_prun_2 = 0;
        M_22_prun_3 = 0;
        M_22_prun_4 = 0;
        M_22_prun_5 = 0;
        M_22_prun = [M_22_prun, [M_11_prun_1; M_22_prun_2; M_22_prun_3; M_22_prun_4; M_22_prun_5]];
    end
    
else%not done enough iterations    
    M_11_prun_2 = 0;
    M_11_prun_3 = 0;
    M_11_prun_4 = 0;
    M_11_prun_5 = 0;
    M_11_prun = [M_11_prun, [M_11_prun_1; M_11_prun_2; M_11_prun_3; M_11_prun_4; M_11_prun_5]];

    M_12_prun_2 = 0;
    M_12_prun_3 = 0;
    M_12_prun_4 = 0;
    M_12_prun_5 = 0;       
    M_12_prun = [M_12_prun, [M_11_prun_1; M_12_prun_2; M_12_prun_3; M_12_prun_4; M_12_prun_5]];
        
    M_21_prun_2 = 0;
    M_21_prun_3 = 0;
    M_21_prun_4 = 0;
    M_21_prun_5 = 0;
    M_21_prun = [M_21_prun, [M_11_prun_1; M_21_prun_2; M_21_prun_3; M_21_prun_4; M_21_prun_5 ]];
    
    M_22_prun_2 = 0;
    M_22_prun_3 = 0;
    M_22_prun_4 = 0;
    M_22_prun_5 = 0;
    M_22_prun = [M_22_prun, [M_11_prun_1; M_22_prun_2; M_22_prun_3; M_22_prun_4; M_22_prun_5]];    
end


M = [M_11_est M_12_est; M_21_est M_22_est];


end
