clc
clear all
close all

x = -1:0.05:1;
y = x;
[X,Y] = meshgrid(x,y);
z = max(exp(-10*X.^2), max(exp(-50*Y.^2),  1.25*exp(-5*(X.^2 + Y.^2))));

figure(1);
surf(x,y,z);
title(['Figure to be estimated']);
xlabel('x axis');
ylabel('y axis');
zlabel('z axis');

%Initialize the RBF network
%RBFN(indim, outdim, min1, max1, step1, min2, max2, step2, ...)
fa = MRL.RBFN(2, 1, -1.0, 1.0, 0.01, -1.0, 1.0, 0.01);

%----------changeabel values

ndata = 20000; %number of training data samples for retraining phases
ndata_final = 30000; %number of training samples for the last training
niterations = 3; %number of pruning and retraining periods
th_pro_start =  0.00015; %starting threshold for weight based pruning in percent (decimal)
outpTh = 0.075; %threshold for node output pruning in percent (decimal)

%----------end changeable values

normOut = zeros(length(fa.mu),1);%initialize the activation function output ratio sum
nInp = 2; %number of inputs
ndata_1 = 50000; %only for the first training period (large enough so that rmse<0.01 
len = length(x)*length(y);
samples_1 = ceil(len*rand(ndata_1,1)); %only for the first training period
samples = ceil(len*rand(ndata,1));%retraining
rms_error = 1;%initializing approximation error

for a = 1:niterations 
       
    if a == niterations
        ndata = ndata_final;
        samples = ceil(len*rand(ndata,1));
    end


    %initial training of the network
    if a == 1
        i = 0;
        while(rms_error > 0.01)
            i = i + 1;

            [xval yval] = ind2sub(size(X), samples_1(i));
            [z_est outN] = fa.Calculate([x(xval) y(yval)]);  
            err = z(xval,yval) - z_est;
            fa = fa.Train([x(xval) y(yval)], err);
            
            normOut = normOut + outN;

            if mod(i,1000) == 0
                disp(['Iteration ' num2str(i)]);
                x_est = -1:0.05:1;
                y_est = x_est;
                z_est = zeros(length(x_est), length(y_est));
                for k = 1:length(x_est),
                    for l = 1:length(y_est),
                        z_est(k,l) = fa.Calculate([x_est(k) y_est(l)]);
                    end
                end
                dim_zest = size(z_est);
                rms_error = sqrt(sum(sum((z_est - z).^2))/(dim_zest(1)*dim_zest(2)));
            end    
        end %end first training, while loop

        figure;
        surf(x_est, y_est, z_est);
        title(['Function before pruning']);
        xlabel('x axis');
        ylabel('y axis');
        zlabel('z axis');
        
        th_pro = th_pro_start; %first weight pruning threshold
        disp(['approximation error before pruning is ' num2str(rms_error)]);
    end %if a == 1 -----------------------------------------------

    disp(['before removing - centers:  ' num2str(length(fa.mu)) ]);
    disp(['starts with pruning, time number ' num2str(a) ]);
    weight = fa.w;
    iterations = length(weight);
    weight_norm = norm(weight,2);
    j = 0;
    m = 0;
    min_size = (weight_norm * th_pro); %actual threshold for weights based pruning
    prun_normOut = outpTh * max(normOut); %actual threshold for node output pruning
    small_weights = zeros(iterations,1);
    small_output = zeros(iterations,1);
    
    %finds the neurons that are to be pruned by activation outout ratio
    for i = 1 : iterations       
        if normOut(i) < prun_normOut
            m = m+1;
            small_output(m) = i;
        end
    end
    %removes the found neurons 
    for k = 1 : m  
        if k == 1
            index = small_output(k);
        end
        if k > 1
            index = small_output(k) - (k - 1);
        end                     
       fa = fa.remove_neuron(index, nInp);
    end 
    
    weight = fa.w;
    iterations = length(weight);
    %finds which weights to remove
    for i = 1 : iterations
        if abs(weight(i)) < min_size 
            j = j + 1;
            small_weights(j) = i;                        
        end
    end
    %remove the found weights
    for k = 1 : j 
        if k == 1
            index_w = small_weights(k);
        end
        if k > 1
            index_w = small_weights(k) - (k - 1);
        end                     
        fa = fa.remove_neuron(index_w, nInp);
    end 
    
    disp(['number of weights removed is ' num2str(m) ' by output, and ' num2str(j) ' by weight pruning' ]);

    %sets all the norm Outp to zero
    normOut = zeros(length(fa.mu),1);
    
    %retraining the network
    for i = 1:length(samples),
        [xval yval] = ind2sub(size(X), samples(i));
        [z_est_1 outN] = fa.Calculate([x(xval) y(yval)]);
        err = z(xval,yval) - z_est_1;
        fa = fa.Train([x(xval) y(yval)], err);
        normOut = normOut + outN;
        
        if mod(i,1000) == 0,
            disp(['Iteration ' num2str(i)]);
        end 
    end 
    
    disp(['Done with retraining number ' num2str(a)]);

    % Draw the function approximation result 
    x_est_p = -1:0.05:1;
    y_est_p = x_est_p;
    z_est_p = zeros(length(x_est_p), length(y_est_p));
    for i = 1:length(x_est_p),
        for j = 1:length(y_est_p),
            z_est_p(i,j) = fa.Calculate([x_est_p(i) y_est_p(j)]);
        end
    end
    figure;
    surf(x_est_p, y_est_p, z_est_p);
    title(['Figure after pruning']);
    xlabel('x axis');
    ylabel('y axis');
    zlabel('z axis');

    %finding the rms error for the pruned version    
    t2 = size(z_est_p);
    rms_pruned = sqrt(sum(sum((z_est_p - z).^2))/(t2(1)*t2(2)));
    disp(['the rms error after pruning is ' num2str(rms_pruned)]);

    %increasing the weight pruning threshold
    th_pro = th_pro_start*a;
    
end %end the whole thing

disp(['Done! and the final size is ' num2str(length(fa.mu))]);
